#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import re


class SortHelper():

    def __init__(self):
        self.TAG = "budgie-app-launcher.SortHelper"

    def convert(self, text):
        if text.isdigit():
            return int(text)
        else:
            return text.lower()

    def naturalSortAppsByName(self, app):
        key = app.getName()
        cList = []
        if key is not None:
            for c in re.split('([0-9]+)', key):
                cList.append(self.convert(c))
        return cList

    def sortedAppsByName(self, listToSort):
        return sorted(listToSort, key=self.naturalSortAppsByName)

    def naturalSortAppsByIndex(self, app):
        key = str(app.getIndex())
        cList = []
        if key is not None:
            for c in re.split('([0-9]+)', key):
                cList.append(self.convert(c))
        return cList

    def sortedAppsByIndex(self, listToSort):
        return sorted(listToSort, key=self.naturalSortAppsByIndex)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class ArrowButton(Gtk.Button):

    def __init__(self, arrow_direction):

        Gtk.Button.__init__(self)
        self.get_style_context().add_class(Gtk.STYLE_CLASS_FLAT)

        image = Gtk.Image()

        if arrow_direction is Gtk.PositionType.TOP:
            image.set_from_icon_name("go-up-symbolic", Gtk.IconSize.MENU)
            self.add(image)
        elif arrow_direction is Gtk.PositionType.BOTTOM:
            image.set_from_icon_name("go-down-symbolic", Gtk.IconSize.MENU)
            self.add(image)



#!/usr/bin/env python3
import gi
gi.require_version("Gtk", "3.0")
gi.require_version("Wnck", "3.0")
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio, Wnck, GLib
import os
import dropby_tools as db
import subprocess


"""
DropBy
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


class BudgieDropBy(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieDropBy"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieDropByApplet(uuid)


class DropBySettings(Gtk.Grid):

    def __init__(self, setting):

        super().__init__()
        explanation = Gtk.Label()
        explanation.set_text(
            "The applet will show up when a usb device is connected."
        )
        explanation.set_xalign(0)
        explanation.set_line_wrap(True)
        self.attach(explanation, 0, 0, 1, 1)

        self.attach(Gtk.Label(""), 0, 1, 1, 1)
        popup_positionlabel = Gtk.Label()
        popup_positionlabel.set_text(
            "Dropby popup window position:"
        )
        popup_positionlabel.set_xalign(0)
        self.attach(popup_positionlabel, 0, 2, 1, 1)
        self.attach(Gtk.Label(""), 0, 3, 1, 1)
        # settings
        self.settings = Gio.Settings.new(
            "org.ubuntubudgie.plugins.budgie-dropby"
        )
        initially_active = self.settings.get_int("popup-corner")
        # anchor section
        anchorgrid = Gtk.Grid()
        leftspace = Gtk.Label("\t")
        centerlabel = Gtk.Label("\t")
        anchorgrid.attach(leftspace, 1, 0, 1, 1)
        anchorgrid.attach(centerlabel, 3, 1, 1, 1)
        # group
        nw = Gtk.RadioButton(group=None)
        ne = Gtk.RadioButton(group=nw)
        se = Gtk.RadioButton(group=nw)
        sw = Gtk.RadioButton(group=nw)
        self.buttons = [nw, ne, sw, se]
        self.buttons[initially_active - 1].set_active(True)
        pos = [[2, 0], [4, 0], [2, 2], [4, 2]]
        i = 0
        for button in self.buttons:
            button.connect("toggled", self.update_corner)
            anchorgrid.attach(button, pos[i][0], pos[i][1], 1, 1)
            i = i + 1
        self.attach(anchorgrid, 0, 5, 2, 1)
        self.show_all()

    def update_corner(self, togglebutton):
        newcorner = self.buttons.index(togglebutton) + 1
        self.settings.set_int("popup-corner", newcorner)


class BudgieDropByApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """
    manager = None

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        self.connect("destroy", Gtk.main_quit)
        app_path = os.path.dirname(os.path.abspath(__file__))
        self.tmp_path = os.getenv("XDG_RUNTIME_DIR") \
            if "XDG_RUNTIME_DIR" in os.environ else os.getenv("HOME")
        self.copytrigger = os.path.join(self.tmp_path, ".dropby_icon_copy")
        self.copying = False
        self.winpath = os.path.join(app_path, "dropover")
        self.box = Gtk.EventBox()
        self.box.connect("button-press-event", self.create_windowtrigger)
        self.icon = Gtk.Image.new_from_icon_name(
            "budgie-dropby-symbolic", Gtk.IconSize.MENU
        )
        self.idle_icon = Gtk.Image.new_from_icon_name(
            "budgie-dropby-idle", Gtk.IconSize.MENU
        )
        self.smallsq_icon = Gtk.Image.new_from_icon_name(
            "budgie-dropbysmallsq-symbolic", Gtk.IconSize.MENU
        )
        self.bigsq_icon = Gtk.Image.new_from_icon_name(
            "budgie-dropbybigsq-symbolic", Gtk.IconSize.MENU
        )
        self.scr = Wnck.Screen.get_default()
        self.box.add(self.icon)
        self.curr_iconindex = 0
        self.add(self.box)
        self.box.show_all()
        self.show_all()
        self.setup_watching()
        GLib.idle_add(self.start_dropover)
        self.refresh_from_idle()

    def set_icon_state(self):
        if not self.copying:
            self.refresh_from_idle()
        else:
            GLib.timeout_add_seconds(1, self.toggle_icons)

    def toggle_icons(self):
        if self.copying:
            for wdg in self.box.get_children():
                self.box.remove(wdg)
            if self.curr_iconindex == 0:
                nexticon = self.bigsq_icon
                self.curr_iconindex = 1
            elif self.curr_iconindex == 1:
                nexticon = self.idle_icon
                self.curr_iconindex = 2
            elif self.curr_iconindex == 2:
                nexticon = self.smallsq_icon
                self.curr_iconindex = 3
            elif self.curr_iconindex == 3:
                nexticon = self.icon
                self.curr_iconindex = 0
            GObject.idle_add(
                self.update_activeicon, nexticon,
                priority=GObject.PRIORITY_DEFAULT,
            )
        return self.copying

    def update_activeicon(self, icon):
        self.box.add(icon)
        self.box.show_all()
        self.show_all()

    def getridof_copytrigger(self):
        try:
            os.remove(self.copytrigger)
        except FileNotFoundError:
            pass

    def set_iconactive(self, arg1, arg2, arg3, event):
        copytriggerexists = os.path.exists(self.copytrigger)
        copytriggercreated = event == Gio.FileMonitorEvent.CREATED
        if all([copytriggerexists, copytriggercreated]):
            self.copying = True
            self.set_icon_state()
        elif all([self.copying, not copytriggerexists]):
            self.getridof_copytrigger()
            self.copying = False
            self.set_icon_state()

    def create_windowtrigger(self, *args):
        if not self.check_winexists():
            open(os.path.join(self.tmp_path, ".call_dropby"), "wt").write("")

    def start_dropover(self):
        user = os.environ["USER"]
        try:
            pid = subprocess.check_output(
                ["/usr/bin/pgrep", "-u", user, "-f", self.winpath]
            )
        except subprocess.CalledProcessError:
            subprocess.Popen([self.winpath, self.uuid])

    def check_winexists(self):
        wins = self.scr.get_windows()
        for w in wins:
            if w.get_name() == "dropby_popup":
                return True
        return False

    def setup_watching(self):
        # setup watching triggers
        infofile = Gio.File.new_for_path(self.copytrigger)
        try:
            os.remove(self.copytrigger)
        except FileNotFoundError:
            pass
        self.monitor = infofile.monitor(Gio.FileMonitorFlags.NONE, None)
        self.monitor.connect("changed", self.set_iconactive)
        self.watchdrives = Gio.VolumeMonitor.get()
        self.actiontriggers = [
            "volume_added", "volume_removed", "mount_added", "mount_removed",
        ]
        for t in self.actiontriggers:
            self.watchdrives.connect(t, self.refresh_from_idle)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return DropBySettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True

    def refresh_from_idle(self, subject=None, newvol=None):
        GObject.idle_add(
            self.refresh, subject, newvol,
            priority=GObject.PRIORITY_DEFAULT,
        )

    def refresh(self, subject=None, newvol=None):
        allvols = self.watchdrives.get_volumes()
        get_relevant = db.get_volumes(allvols)
        # decide if we should show or not
        for c in self.box.get_children():
            c.destroy()
        if get_relevant:
            self.box.add(self.icon)
        else:
            for img in self.box:
                self.box.remove(img)
        self.box.show_all()



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import gi.repository

gi.require_version('Budgie', '1.0')
from gi.repository import Budgie
from gi.repository import GObject
from AppLauncherApplet import AppLauncherApplet


class AppLauncher(GObject.GObject, Budgie.Plugin):
    # This is simply an entry point into your Budgie Applet implementation.
    # Note you must always override Object, and implement Plugin.

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "io_serdarsen_github_budgie_app_launcher"

    def __init__(self):
        # Initialisation is important.
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        # This is where the real fun happens. Return a
        # new Budgie.Applet instance with the given UUID.
        # The UUID is determined by the BudgiePanelManager, and is used for
        # lifetime tracking.
        return AppLauncherApplet(uuid)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class MenuButton(Gtk.HBox):

    def __init__(self, app, iconSize):
        Gtk.HBox.__init__(self)

        self.app = app
        info = self.app.getInfo()  # DesktopAppInfo
        icon = info.get_icon()
        if icon is not None:
            img = Gtk.Image.new_from_gicon(icon, Gtk.IconSize.INVALID)
            img.set_pixel_size(iconSize)
            img.margin_end = 7
        else:
            img = None
        lab = Gtk.Label(info.get_display_name(), xalign=0)
        lab.set_markup(
            "<span size='small'>%s</span>" % info.get_display_name().replace(
                "&", "&amp;"))
        lab.set_margin_left(7)
        lab.halign = Gtk.Align.START
        lab.valign = Gtk.Align.CENTER
        layout = Gtk.Box(Gtk.Orientation.HORIZONTAL, 0)
        if img:
            layout.pack_start(img, False, False, 0)
        layout.pack_start(lab, True, True, 0)
        self.toggleButton = Gtk.ToggleButton()
        self.pack_start(self.toggleButton, True, True, 0)
        self.toggleButton.get_style_context().add_class("flat")
        self.toggleButton.set_sensitive(False)
        self.toggleButton.add(layout)
        self.set_tooltip_text(info.get_description())
        self.checkButton = Gtk.CheckButton()
        self.pack_end(self.checkButton, False, False, 0)
        self.checkButton.set_margin_left(7)
        self.checkButton.set_size_request(24, 24)
        self.checkButton.show_all()

    def addOnToggleMethod(self, method):
        self.toggleButton.connect("toggled", method, self)

    def addOnCheckMethod(self, method):
        self.checkButton.connect("toggled", method, self)

    def setToggled(self, isActive):
        self.toggleButton.set_active(isActive)

    def setToggButtonSensitive(self, isActive):
        self.toggleButton.set_sensitive(isActive)

    def getToggled(self, isActive):
        return self.toggleButton.get_active()

    def setChecked(self, isActive):
        self.checkButton.set_active(isActive)

    def getChecked(self):
        return self.checkButton.get_active()

    def getApp(self):
        return self.app



#!/usr/bin/env python3
import pyudev
import psutil
import subprocess
import time


"""
DropBy
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <https://www.gnu.org/licenses/>.
"""


def bytes_toreadable(bte):
    """
    convert b (float/int) into the most convenient unit (str)
    """
    unit_names = ["KB", "MB", "GB", "TB"]
    units = [1024**q for q in range(4)]
    numbers = [d for d in [bte / (n * 1024) for n in units] if d >= 1]
    try:
        return str(round(numbers[-1], 1)) + " " + unit_names[len(numbers) - 1]
    except IndexError:
        return "0 KB"


def get_usb():
    """
    get usb devices by uuid (pyudev)
    """
    relevant = []
    udev_context = pyudev.Context()
    allvols = udev_context.list_devices(subsystem='block')
    for v in allvols:
        try:
            uuid = v.get('ID_FS_UUID')
        except Exception:
            pass
        else:
            devpath = v.get("DEVPATH")
            if devpath and uuid and any(["usb" in devpath, "mmc" in devpath]):
                relevant.append(uuid)
    return relevant


def get_mounted():
    """
    get info on *mounted* volumes (psutil)
    device path, mountpoint, usage
    """
    relevant = []
    mounted = psutil.disk_partitions(all=False)
    for v in mounted:
        usage = None
        try:
            usage = psutil.disk_usage(v.mountpoint)
        except (PermissionError, OSError):
            continue
        dev = v.device
        # mountpoint is relevant for possible actions
        mpoint = v.mountpoint
        try:
            us = usage.free
        except AttributeError:
            us = "--"
        relevant.append([dev, mpoint, us])
    return relevant


def uuid_todev(uuid):
    try:
        return subprocess.check_output(
            ["/usr/sbin/findfs", "UUID=" + uuid]
        ).decode("utf-8").strip()
    except FileNotFoundError:
        try:
            return subprocess.check_output(
                ["/sbin/findfs", "UUID=" + uuid]
            ).decode("utf-8").strip()
        except (subprocess.CalledProcessError, TypeError):
            return None
    except (subprocess.CalledProcessError, TypeError):
        return None


def get_volumes(allvols):
    """
    get relevant data on all usb volumes
    """
    relevant = []
    # gather data
    usb_devs = get_usb()
    use_data = get_mounted()
    mounted_devspaths = [d[0] for d in use_data]
    # filter out relevants
    for v in allvols:
        # filter out usb devices
        uuid = v.get_uuid()
        if uuid in usb_devs:
            path = uuid_todev(uuid)
            devdata = {
                "volume": v,
                "name": v.get_name(),
                "uuid": uuid,
                "device": path,
                "can_mount": v.can_mount(),
                "icon": v.get_icon(),
                "flashdrive": v.can_eject(),
                "ismounted": v.get_mount(),
            }
            # try to determine usage
            try:
                match = mounted_devspaths.index(path)
            except ValueError:
                free = ""
                fpath = None
            else:
                free = bytes_toreadable(use_data[match][2])
                fpath = use_data[match][1]
            devdata["free"] = free
            devdata["volume_path"] = fpath
            relevant.append(devdata)
    return (relevant)



#! /usr/bin/python3

"""
Keyboard Auto Switch
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""


import os
import gi
gi.require_version('Budgie', '1.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Budgie, GObject, Gtk, Gio, GLib
from threading import Thread
import time
import subprocess
import ast
import psutil
import dbus


settingspath = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "langswitch",
)


default_langfile = os.path.join(settingspath, "default_lang")
lang_datafile = os.path.join(settingspath, "lang_data")


try:
    os.makedirs(settingspath)
except FileExistsError:
    pass


class KeyboardAutoSwitch(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "KeyboardAutoSwitch"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return KeyboardAutoSwitchApplet(uuid)


class KeyboardAutoSwitchApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        # setup watching applet presence
        self.currpanelsubject_settings = None
        GLib.timeout_add_seconds(1, self.watchout)
        self.kbautoswitch_onpanel = True
        # general stuff
        self.key = "org.gnome.desktop.input-sources"
        self.settings = Gio.Settings.new(self.key)
        # menugrid
        self.menugrid = Gtk.Grid()
        self.menugrid.set_row_spacing(5)
        self.menugrid.set_column_spacing(20)
        # left space
        self.menugrid.attach(Gtk.Label("   "), 0, 0, 1, 1)
        # Default language section
        self.menugrid.attach(Gtk.Label(
            "Default layout:", xalign=0), 1, 1, 1, 1
        )
        self.langlist_combo = Gtk.ComboBoxText()
        self.langlist_combo.set_entry_text_column(0)
        self.langlist_combo.set_size_request(185, 20)
        self.menugrid.attach(self.langlist_combo, 1, 2, 1, 1)
        self.menugrid.attach(
            Gtk.Label("\nExceptions: ", xalign=0), 1, 4, 1, 1
        )
        # Exceptions section
        self.exc_combo = Gtk.ComboBoxText()
        self.exc_combo.set_entry_text_column(0)
        self.menugrid.attach(self.exc_combo, 1, 5, 1, 1)
        delete_button = Gtk.Button()
        delete_img = self.seticon = Gtk.Image.new_from_icon_name(
            "user-trash-symbolic", Gtk.IconSize.MENU
        )
        delete_button.set_image(delete_img)
        self.menugrid.attach(delete_button, 2, 5, 1, 1)
        # end spacer
        spacer_end = Gtk.Label("")
        self.menugrid.attach(spacer_end, 3, 10, 1, 1)
        # panel
        self.seticon = Gtk.Image.new_from_icon_name(
            "budgie-keyboard-autoswitch-symbolic", Gtk.IconSize.MENU
        )
        self.box = Gtk.EventBox()
        self.box.add(self.seticon)
        self.add(self.box)
        self.popover = Budgie.Popover.new(self.box)
        self.popover.add(self.menugrid)
        self.popover.get_child().show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)
        # initiate
        try:
            # get the possible existing dict data
            f = open(lang_datafile)
            val = f.read()
            if val == "":
                raise FileNotFoundError
            self.langdata = ast.literal_eval(val.strip())

        except FileNotFoundError:
            self.langdata = {}
        try:
            # get the possible previously set (and saved) default language
            self.default_lang = open(default_langfile).read().strip()
        except FileNotFoundError:
            lang_index = self.settings.get_uint("current")
            self.default_lang = self.readable_lang(
                self.settings.get_value("sources")[lang_index][1]
            )
        self.langlist_selection_id = self.langlist_combo.connect(
            "changed", self.change_ondeflang_select,
        )
        delete_button.connect("clicked", self.remove_exception)
        self.act_on_gsettingschange()
        self.settings.connect("changed::sources", self.act_on_gsettingschange)

        # Use dbus connection to check for the screensaver activity
        self.bus = dbus.SessionBus()
        self.screensaver = None

        # thread
        GObject.threads_init()
        # thread
        self.update = Thread(target=self.watch_yourlanguage)
        # daemonize the thread to make the indicator stopable
        self.update.setDaemon(True)
        self.update.start()

    def watchout(self):
        path = "com.solus-project.budgie-panel"
        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
        panel_settings = Gio.Settings.new(path)
        allpanels_list = panel_settings.get_strv("panels")
        for p in allpanels_list:
            panelpath = panelpath_prestring + "{" + p + "}/"
            self.currpanelsubject_settings = Gio.Settings.new_with_path(
                path + ".panel", panelpath
            )
            applets = self.currpanelsubject_settings.get_strv("applets")
            if self.uuid in applets:
                self.currpanelsubject_settings.connect(
                    "changed", self.check_ifonpanel
                )
        return False

    def check_ifonpanel(self, *args):
        applets = self.currpanelsubject_settings.get_strv("applets")
        self.kbautoswitch_onpanel = self.uuid in applets

    def on_press(self, box, arg):
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)

    def readable_lang(self, lang):
        lang = lang.split("+")
        try:
            lang[1] = "(" + lang[1] + ")"
        except IndexError:
            return lang[0]
        else:
            return " ".join(lang)

    def lockscreen_check(self):
        try:
            if self.screensaver is None:
                self.screensaver = self.bus.get_object("org.gnome.ScreenSaver",
                                                       "/")
            val = self.screensaver.get_dbus_method('GetActive')
            return val()
        except dbus.exceptions.DBusException:
            self.screensaver = None
            return False

    def change_ondeflang_select(self, widget):
        """
        change the default language, update settings file and exceptions list
        """
        self.default_lang = self.langlist_combo.get_active_text()
        open(default_langfile, "wt").write(str(self.default_lang))
        self.clear_deflangkey()
        open(lang_datafile, "wt").write(str(self.langdata))
        self.update_exceptions_gui()

    def clear_deflangkey(self):
        """
        if th newly set default language has any exceptions, they should be
        cleared. there is no point in setting classes as exceptions (anymore)
        then. this function is called on change of the default language from
        the menu.
        """
        # list keys
        keys = list(self.langdata.keys())
        # find index of default lang dict-item
        sub = [
            self.langdata[k]["readable"] for k in keys
        ].index(self.default_lang)
        key = keys[sub]
        # clear the exceptions list of the default language
        self.langdata[key]["classes"] = []

    def remove_wmclass(self, wmclass):
        """
        finds occurrences of wmclass in the lang_data, removes them
        """
        keys = list(self.langdata.keys())
        sub = [self.langdata[k]["classes"] for k in keys]
        for s in sub:
            if wmclass in s:
                index = sub.index(s)
                self.langdata[keys[sub.index(s)]]["classes"].remove(wmclass)
                break
        open(lang_datafile, "wt").write(str(self.langdata))

    def update_langlist_gui(self):
        """
        update the list of languages, as it appears in "Default language"
        -options.
        """
        # disconnect
        self.langlist_combo.disconnect(self.langlist_selection_id)
        # why not fetch from self.raw_langlist?
        readable_list = []
        # delete all entries
        self.langlist_combo.remove_all()
        # add to readable list temporarily to determine index to set
        for k in self.langdata.keys():
            name = self.langdata[k]["readable"]
            readable_list.append(name)
        # add all languages to gui
        for n in readable_list:
            self.langlist_combo.append_text(n)
        # find index
        try:
            index = readable_list.index(self.default_lang)
        except ValueError:
            index = 0
        self.default_lang = readable_list[index]
        open(default_langfile, "wt").write(str(self.default_lang))
        self.langlist_combo.set_active(index)
        # set the connection again
        self.langlist_selection_id = self.langlist_combo.connect(
            "changed", self.change_ondeflang_select,
        )

    def act_on_gsettingschange(self, *args):
        """
        fetch current languages, update language data (dict), remove
        obsolete langs, add new ones. then save the new dict to file.
        """
        # fetch current languages from gsettings
        self.raw_langlist = [
            item[1] for item in self.settings.get_value("sources")
        ]
        # add new languages
        curr_keys = list(self.langdata.keys())
        for sl in self.raw_langlist:
            readable = self.readable_lang(sl)
            if sl not in curr_keys:
                self.langdata[sl] = {"classes": [], "readable": readable}
        # remove obsolete languages + data
        for k in curr_keys:
            if k not in self.raw_langlist:
                del self.langdata[k]
        open(lang_datafile, "wt").write(str(self.langdata))
        self.update_langlist_gui()

    def find_exception(self, wmclass):
        """
        search self.langdata for existing exceptions
        """
        keys = self.langdata.keys()
        exist = None
        for k in keys:
            data = self.langdata[k]["classes"]
            if wmclass in data:
                exist = k
                break
        # output = raw lang!
        return exist

    def set_lang_onclasschange(self, wmclass, lang):
        """
        if the wmclass changes (window change), check if an exception exists
        on the wmclass. switch language if lang is not the currently active one
        """
        curr_exception = self.find_exception(wmclass)
        if curr_exception:
            # if the window is an exception, *and* another language; set lang
            if lang != curr_exception:
                self.set_newlang(newlang=curr_exception)
        elif self.readable_lang(lang) != self.default_lang:
            self.set_newlang(default=True)

    def set_newlang(self, newlang=None, default=False):
        """
        switch source for the currently active window
        """
        if newlang:
            index = self.raw_langlist.index(newlang)
        elif default:
            getreadables = [self.readable_lang(sl) for sl in self.raw_langlist]
            index = getreadables.index(self.default_lang)
        self.settings.set_uint("current", index)

    def lock_state(self, oldlang):
        while True:
            time.sleep(1)
            if not self.lockscreen_check() or not self.kbautoswitch_onpanel:
                break
        self.set_newlang(oldlang)

    def watch_yourlanguage(self):
        # fill exceptions (gui) list with data
        self.update_exceptions_gui()
        # fetch set initial data
        wmclass1 = self.get_activeclass()
        activelang1 = self.get_currlangname()
        while self.kbautoswitch_onpanel:
            time.sleep(1)
            # if language is changed during lockstate, revert afterwards
            if self.lockscreen_check():
                self.lock_state(activelang1)
            wmclass2 = self.get_activeclass()
            activelang2 = self.get_currlangname()
            # first set a few conditions to act *at all*
            if all(
                [wmclass2, wmclass2 != "raven",
                 wmclass2 != "Wprviews_window",
                 activelang2]):
                classchange = wmclass2 != wmclass1
                langchange = activelang2 != activelang1
                if classchange:
                    self.set_lang_onclasschange(wmclass2, activelang2)
                    activelang2 = self.get_currlangname()
                elif langchange:
                    self.set_exception(activelang2, wmclass2)
                    GObject.idle_add(
                        self.update_exceptions_gui,
                        priority=GObject.PRIORITY_DEFAULT,
                    )
                    open(lang_datafile, "wt").write(str(self.langdata))
                wmclass1 = wmclass2
                activelang1 = activelang2

    def update_exceptions_gui(self):
        self.exc_combo.remove_all()
        keys = list(self.langdata.keys())
        for k in keys:
            wmclasses = self.langdata[k]["classes"]
            for cl in wmclasses:
                mention = ", ".join([cl, k])
                self.exc_combo.append_text(mention)
        self.exc_combo.set_active(0)

    def remove_exception(self, button):
        """
        remove an exception from the menu (gui)
        """
        try:
            toremove = self.exc_combo.get_active_text().split(", ")[0]
        except AttributeError:
            pass
        else:
            self.remove_wmclass(toremove)
            self.update_exceptions_gui()

    def set_exception(self, lang, wmclass):
        lang = self.readable_lang(lang)
        # remove possible existing exception
        self.remove_wmclass(wmclass)
        # add new exception
        keys = list(self.langdata.keys())
        sub = [self.langdata[k]["readable"] for k in keys].index(lang)
        if lang != self.default_lang:
            self.langdata[keys[sub]]["classes"].append(wmclass)

    def get(self, cmd):
        try:
            return subprocess.check_output(cmd).decode("utf-8").strip()
        except subprocess.CalledProcessError:
            pass

    def show_wmclass(self, wid):
        # handle special cases
        try:
            cl = self.get([
                "/usr/bin/xprop", "-id", wid, "WM_CLASS"
            ]).split("=")[-1].split(",")[-1].strip().strip('"')
        except (IndexError, AttributeError):
            pass
        else:
            # exceptions; one application, multiple WM_CLASS
            if "Thunderbird" in cl:
                return "Thunderbird"
            elif "Toplevel" in cl:
                return "Toplevel"
            else:
                return cl

    def get_activeclass(self):
        # get WM_CLASS of active window
        currfront = self.get(["/usr/bin/xdotool", "getactivewindow"])
        return self.show_wmclass(currfront) if currfront else None

    def get_currlangname(self):
        i = self.settings.get_uint("current")
        try:
            return self.raw_langlist[i]
        except IndexError:
            pass



#! /usr/bin/python3
import os
import gi
gi.require_version('Budgie', '1.0')
gi.require_version('Wnck', '3.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Budgie, GObject, Gtk, Wnck, Gio, GLib
import time
import ast


"""
Budgie Workspace Timer
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""


class BudgieWorkspaceStopwatch(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieWorkspaceStopwatch"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieWorkspaceStopwatchApplet(uuid)


class BudgieWorkspaceStopwatchApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        # for exit-from-panel procedure
        self.uuid = uuid
        self.currpanelsubject_settings = None
        self.wstopwatch_onpanel = True
        GLib.timeout_add_seconds(1, self.watchout)
        # setup css
        timer_css = """
        .label {
          padding-bottom: 7px;
          padding-top: 0px;
          font-weight: bold;
        }
        .button {
          margin-top: 10px;
          margin-left: 30px;
        }
        """
        self.provider = Gtk.CssProvider.new()
        self.provider.load_from_data(timer_css.encode())
        # setup general stuff
        self.scr = Wnck.Screen.get_default()
        # self.scr.force_update()
        self.scr.connect("active-workspace-changed", self.act_on_change)
        self.logfile = os.path.join(os.environ["HOME"], ".workspace_log")
        self.load_data()
        currws = self.scr.get_active_workspace()
        self.starttime = time.time()
        self.last_logged = self.starttime
        self.act_on_change(self.scr, currws)
        GLib.timeout_add_seconds(30, self.update_log)
        self.maingrid = Gtk.Grid()
        # panel
        self.seticon = Gtk.Image.new_from_icon_name(
            "budgie-wstopwatch-symbolic", Gtk.IconSize.MENU
        )
        self.box = Gtk.EventBox()
        self.box.add(self.seticon)
        self.add(self.box)
        self.popover = Budgie.Popover.new(self.box)
        self.popover.add(self.maingrid)
        self.popover.get_child().show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)

    def watchout(self):
        path = "com.solus-project.budgie-panel"
        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
        panel_settings = Gio.Settings.new(path)
        allpanels_list = panel_settings.get_strv("panels")
        for p in allpanels_list:
            panelpath = panelpath_prestring + "{" + p + "}/"
            self.currpanelsubject_settings = Gio.Settings.new_with_path(
                path + ".panel", panelpath
            )
            applets = self.currpanelsubject_settings.get_strv("applets")
            if self.uuid in applets:
                self.currpanelsubject_settings.connect(
                    "changed", self.check_ifonpanel
                )
        return False

    def check_ifonpanel(self, *args):
        applets = self.currpanelsubject_settings.get_strv("applets")
        self.wstopwatch_onpanel = self.uuid in applets

    def update_log(self):
        self.newlogged = time.time()
        if self.newlogged - self.last_logged > 35:
            currws = self.scr.get_active_workspace()
            self.starttime = time.time()
            self.act_on_change(self.scr, currws)
        open(self.logfile, "wt").write(str(self.workspace_data))
        self.last_logged = self.newlogged
        return self.wstopwatch_onpanel

    def load_data(self):
        try:
            self.workspace_data = ast.literal_eval(open(self.logfile).read())
        except (FileNotFoundError, SyntaxError):
            self.workspace_data = {}

    def time_format(self, s):
        # convert time format from seconds to h:m:s
        m, s = divmod(s, 60)
        h, m = divmod(m, 60)
        return "%02d:%02d:%02d" % (h, m, s)

    def act_on_change(self, screen, workspace):
        self.workspaces = screen.get_workspaces()
        key = self.workspaces.index(workspace)
        currtime = time.time()
        span = currtime - self.starttime
        # try get current time for key, add key if needed
        try:
            curr_spent = self.workspace_data[key]["time"]
        except KeyError:
            curr_spent = 0
            self.workspace_data[key] = {
                "time": curr_spent,
                "custom_name": "workspace: " + str(key + 1)
            }
        self.workspace_data[key]["time"] = curr_spent + span
        self.starttime = currtime
        open(self.logfile, "wt").write(str(self.workspace_data))

    def on_press(self, box, arg):
        self.show_result()
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)

    def set_widgetstyle(self, widget, style):
        widget_cont = widget.get_style_context()
        widget_cont.add_class(style)
        Gtk.StyleContext.add_provider(
            widget_cont,
            self.provider,
            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION,
        )

    def show_result(self, *args):
        self.maingrid.destroy()
        self.maingrid = Gtk.Grid()
        self.popover.add(self.maingrid)
        # update to latest
        currws = self.scr.get_active_workspace()
        self.act_on_change(self.scr, currws)
        topleft = Gtk.Label()
        topleft.set_text("\t")
        self.maingrid.attach(topleft, 0, 0, 1, 1)
        bottomright = Gtk.Label()
        bottomright.set_text("\t")
        self.maingrid.attach(bottomright, 100, 100, 1, 1)
        workspace_header = Gtk.Label()
        workspace_header.set_text("Workspace")
        self.maingrid.attach(workspace_header, 2, 1, 1, 1)
        workspace_header.set_xalign(0)
        time_header = Gtk.Label()
        time_header.set_text("Time")
        self.maingrid.attach(time_header, 4, 1, 1, 1)
        time_header.set_xalign(0)
        for label in [workspace_header, time_header]:
            self.set_widgetstyle(label, "label")
        n = 2
        for k in sorted(self.workspace_data.keys()):
            if n - 2 == self.workspaces.index(currws):
                bullet = Gtk.Label()
                bullet.set_text("⮕ ")
                self.maingrid.attach(bullet, 1, n, 1, 1)
            entry = Gtk.Entry()
            entry.set_text(self.workspace_data[k]["custom_name"])
            entry.connect("changed", self.update_customname, k)
            self.maingrid.attach(entry, 2, n, 1, 1)
            spacer = Gtk.Label()
            spacer.set_text("\t")
            self.maingrid.attach(spacer, 3, n, 1, 1)
            timelabel = Gtk.Label()
            timelabel.set_text(
                str(self.time_format(int(self.workspace_data[k]["time"])))
            )
            timelabel.set_xalign(0)
            self.maingrid.attach(timelabel, 4, n, 1, 1)
            n = n + 1
        resetbutton = Gtk.Button.new_with_label("Reset")
        resetbutton.grab_focus()
        resetbutton.connect("clicked", self.reset_data)
        self.set_widgetstyle(resetbutton, "button")
        self.maingrid.attach(
            resetbutton, 4, 99, 1, 1
        )
        resetbutton.grab_focus()
        self.maingrid.show_all()
        self.popover.show_all()

    def reset_data(self, button):
        self.workspaces = self.scr.get_workspaces()
        todeletekeys = []
        for k in self.workspace_data.keys():
            if k >= len(self.workspaces):
                todeletekeys.append(k)
            else:
                self.workspace_data[k]["time"] = 0
        for k in todeletekeys:
            del self.workspace_data[k]
        self.show_result()

    def update_customname(self, entry, key):
        newname = entry.get_text()
        self.workspace_data[key]["custom_name"] = newname
        self.update_log()



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

import errno
import os

from App import App
from JsonHelper import JsonHelper


class LocaleHelper():

    def __init__(self):

        self.TAG = "LocalStateHelper"
        self.jsonHelper = JsonHelper()

        self.dirPath = os.path.expanduser("~") + "/.config/budgie-app-launcher"
        self.makeDirIfNotExist(self.dirPath)
        self.filePath = self.dirPath + "/Locale State"

    # makes dir
    def makeDirIfNotExist(self, path):
        if (path != ""):
            try:
                os.makedirs(path)
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise

    def retriveApps(self):
        apps = []
        data = self.getData()
        if "apps" in data:
            appsData = data["apps"]
            for appKey in appsData:
                appData = appsData[appKey]
                id = appData["id"]
                name = appData["name"]
                isActive = appData["isActive"]
                index = appData["index"]
                app = App(id, name, isActive)
                app.setIndex(index)
                apps.append(app)

        return apps

    def saveShowOnPanel(self, showOnPanel):
        data = self.getData()
        data["showOnPanel"] = showOnPanel
        self.jsonHelper.writeData(self.filePath, data)

    def retriveShowOnPanel(self):
        showOnPanel = 0
        data = self.getData()
        if "showOnPanel" in data:
            showOnPanel = data["showOnPanel"]
        return showOnPanel

    def saveApps(self, apps):
        data = self.getData()
        appsData = {}
        for app in apps:
            appsData[app.getName()] = \
                {"id": app.getId(),
                 "name": app.getName(),
                 "isActive": app.getActive(),
                 "index": app.getIndex()}

        data["apps"] = appsData
        self.jsonHelper.writeData(self.filePath, data)

    def retriveAppsDict(self):
        appsDict = {}
        data = self.getData()
        if "apps" in data:
            appsData = data["apps"]

            # Converts json data to python object
            for appKey in appsData:
                appData = appsData[appKey]
                id = appData["id"]
                name = appData["name"]
                isActive = appData["isActive"]
                index = appData["index"]
                app = App(id, name, isActive)
                app.setIndex(index)
                appsDict[id] = app
        return appsDict

    def saveIconSize(self, iconSize):
        data = self.getData()
        data["iconSize"] = iconSize
        self.jsonHelper.writeData(self.filePath, data)

    def retriveIconSize(self):
        iconSize = 24
        data = self.getData()
        if "iconSize" in data:
            iconSize = data["iconSize"]
        return iconSize

    def getData(self):
        self.makeDirIfNotExist(self.dirPath)
        data = self.jsonHelper.readData(self.filePath)
        if data is None:
            data = {}
        return data



#! /usr/bin/python3

"""
Count Down
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""


import os
import gi
gi.require_version('Budgie', '1.0')
gi.require_version('Gtk', '3.0')
from gi.repository import Budgie, GObject, GdkPixbuf, Gtk, Gio, GLib
from threading import Thread
import time
import subprocess
import ast


powersettings = Gio.Settings.new(
    "org.gnome.settings-daemon.plugins.power"
)


subs = [
    ["ac", "sleep-inactive-ac-type"],
    ["battery", "sleep-inactive-battery-type"],
]


settingspath = os.path.join(
    os.environ["HOME"], ".config", "budgie-extras", "countdown",
)


try:
    os.makedirs(settingspath)
except FileExistsError:
    pass

# icons
path = "/usr/share/pixmaps"


class CountDown(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieCountDown"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return CountDownApplet(uuid)


class CountDownSettings(Gtk.Grid):

    def __init__(self, setting):

        super().__init__()
        self.show_seconds = True
        self.setting = setting
        # grid & layout
        countdown_spacegrid = Gtk.Grid()
        self.add(countdown_spacegrid)
        for cell in [[0, 0], [100, 0], [0, 100], [100, 100]]:
            countdown_spacegrid.attach(
                Gtk.Label(label="\t"), cell[0], cell[1], 1, 1
            )
        self.space_settings = Gio.Settings(
            schema="org.ubuntubudgie.plugins.budgie-countdown"
        )
        currvalue = self.space_settings.get_int("spacersize")
        space = Gtk.SpinButton()
        space.set_range(0, 50)
        space.set_increments(1, 1)
        space.set_value(currvalue)
        space.connect("value-changed", self.update_value)
        label = Gtk.Label("Built-in spacer" + "\n")
        label.set_xalign(0)
        countdown_spacegrid.attach(label, 1, 1, 2, 1)
        countdown_spacegrid.attach(space, 1, 2, 1, 1)
        self.show_all()

    def update_value(self, spin):
        newval = spin.get_value()
        self.space_settings.set_int("spacersize", newval)


class CountDownApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    def __init__(self, uuid):
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        self.red_time = 60
        self.yellow_time = 300
        # setup watching applet presence
        self.currpanelsubject_settings = None
        GLib.timeout_add_seconds(1, self.watchout)
        self.countdown_onpanel = True
        # spacesettings
        self.space_settings = Gio.Settings(
            schema="org.ubuntubudgie.plugins.budgie-countdown"
        )
        self.claimed_panelspace = self.space_settings.get_int("spacersize")
        self.space_settings.connect("changed", self.get_currclaimedspace)
        # --- panelgrid/containergrid ---
        self.panelgrid = Gtk.Grid()
        self.panelgrid.set_row_spacing(5)
        self.panelgrid.set_column_spacing(5)
        self.containergrid = Gtk.Grid()
        self.containergrid.attach(self.panelgrid, 1, 1, 1, 1)
        # --- let's initiate some stuff ---
        self.panelspacing = 0
        self.position_index = 0
        # label/spacer position
        self.labelpos = [0, 0, 1, 1]
        self.spacerpos = "top"
        self.grid_helpers = []  # spacer images
        self.labelplacement = 0  # distance
        # icons
        grey = os.path.join(path, "cr_grey.png")
        green = os.path.join(path, "cr_green.png")
        yellow = os.path.join(path, "cr_yellow.png")
        red = os.path.join(path, "cr_red.png")
        # pixbuf
        self.iconset = [
            GdkPixbuf.Pixbuf.new_from_file(ic) for ic in
            [red, yellow, green, grey]
        ]
        # initial icon
        self.seticon = Gtk.Image.new_from_pixbuf(self.iconset[0])
        self.panelgrid.attach(self.seticon, 1, 1, 1, 1)
        # menu label
        self.timer = Gtk.Label(label="00:00:00")
        self.panelgrid.attach(self.timer, 2, 1, 1, 1)
        # --- menugrid ---
        self.menugrid = Gtk.Grid()
        self.menugrid.set_column_spacing(15)
        self.menugrid.set_row_spacing(5)
        # left space
        self.menugrid.attach(Gtk.Label(""), 1, 0, 1, 1)
        # hrs
        self.hrs_label = Gtk.Label("Hours: ", xalign=0)
        self.menugrid.attach(self.hrs_label, 1, 2, 1, 1)
        adjustment = Gtk.Adjustment(0, 0, 23, 1)
        self.hoursbutton = Gtk.SpinButton()
        self.hoursbutton.set_adjustment(adjustment)
        self.menugrid.attach(self.hoursbutton, 2, 2, 1, 1)
        # mins
        self.mins_label = Gtk.Label("Minutes: ", xalign=0)
        self.menugrid.attach(self.mins_label, 1, 3, 1, 1)
        adjustment = Gtk.Adjustment(0, 0, 59, 1)
        self.minsbutton = Gtk.SpinButton()
        self.minsbutton.set_adjustment(adjustment)
        self.menugrid.attach(self.minsbutton, 2, 3, 1, 1)
        # secs
        self.secs_label = Gtk.Label("Seconds: ", xalign=0)
        self.menugrid.attach(self.secs_label, 1, 4, 1, 1)
        adjustment = Gtk.Adjustment(0, 0, 59, 1)
        self.secsbutton = Gtk.SpinButton()
        self.secsbutton.set_adjustment(adjustment)
        self.menugrid.attach(self.secsbutton, 2, 4, 1, 1)
        for sp in [self.hoursbutton, self.minsbutton, self.secsbutton]:
            sp.set_numeric(True)
            sp.set_update_policy(True)
        # prevent pause
        self.sleep = Gtk.CheckButton("Prevent pausing countdown")
        self.menugrid.attach(self.sleep, 1, 6, 2, 1)
        sep = Gtk.Separator()
        self.menugrid.attach(sep, 4, 1, 1, 8)
        self.bbox = Gtk.Box()
        self.menugrid.attach(self.bbox, 0, 10, 9, 2)
        self.menugrid.attach(Gtk.Label(""), 1, 9, 1, 1)
        # apply
        self.applybutton = Gtk.Button("Run")
        # set style start/stop
        self.context_start = self.applybutton.get_style_context()
        self.applybutton.set_size_request(80, 20)
        self.bbox.pack_end(self.applybutton, False, False, 0)
        self.applybutton.connect("clicked", self.handle_apply)
        self.nf_bell = Gtk.CheckButton("Ring bell")
        self.menugrid.attach(self.nf_bell, 5, 2, 1, 1)
        self.nf_icon = Gtk.CheckButton("Flash icon")
        self.menugrid.attach(self.nf_icon, 5, 3, 1, 1)
        self.nf_message = Gtk.CheckButton("Display window")
        self.menugrid.attach(self.nf_message, 5, 4, 1, 1)
        self.runcomm = Gtk.CheckButton("Run command:")
        self.menugrid.attach(self.runcomm, 5, 5, 1, 1)
        self.command_entry = Gtk.Entry()
        self.command_entry.connect("key-release-event", self.update_command)
        self.menugrid.attach(self.command_entry, 5, 6, 1, 1)
        # button, file, related variable-key
        self.settingsdata = [
            [self.nf_bell, "mute_ringbell", "ringbell"],
            [self.nf_icon, "mute_flashicon", "flashicon"],
            [self.nf_message, "mute_showwindow", "showwindow"],
            [self.sleep, "mute_nosleep", "keeprun"],
            [self.runcomm, "runcommand", "runcmd"],
        ]
        self.vals = []
        # fetch values for checkbuttons (except command: separate)
        for item in self.settingsdata[:-1]:
            val = True if self.get_setting(item) is False else False
            subj = item[0]
            item[0].set_active(val)
            self.vals.append(val)
            item[0].connect("toggled", self.manage_checkbox)
        # fetch command if True, set checkbutton
        command_set = self.settingsdata[-1]
        commandval = self.get_setting(command_set, True)
        try:
            data = ast.literal_eval(commandval[1])
        except TypeError:
            self.runcomm.set_active(False)
            self.command_entry.set_sensitive(False)
        else:
            self.runcmd = data[0]
            self.command_entry.set_text(data[1])
            self.runcomm.set_active(self.runcmd)
            self.command_entry.set_sensitive(self.runcmd)
        self.runcomm.connect("toggled", self.manage_checkbox)
        self.runvars = {
            "ringbell": self.vals[0],
            "flashicon": self.vals[1],
            "showwindow": self.vals[2],
            "keeprun": self.vals[3],
            "runcmd": self.runcomm.get_active(),
        }
        self.countdown, self.span = 0, 0
        self.cancel = True
        # panel
        self.box = Gtk.EventBox()
        self.box.add(self.containergrid)
        self.add(self.box)
        # menu
        self.popover = Budgie.Popover.new(self.box)
        self.popover.add(self.menugrid)
        self.popover.get_child().show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)
        GObject.threads_init()
        # thread
        self.update = Thread(target=self.run_countdown)
        # daemonize the thread to make the indicator stopable
        self.update.setDaemon(True)
        self.update.start()
        self.seticon.set_from_pixbuf(self.iconset[1])

    def watchout(self):
        path = "com.solus-project.budgie-panel"
        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
        panel_settings = Gio.Settings.new(path)
        allpanels_list = panel_settings.get_strv("panels")
        for p in allpanels_list:
            panelpath = panelpath_prestring + "{" + p + "}/"
            self.currpanelsubject_settings = Gio.Settings.new_with_path(
                path + ".panel", panelpath
            )
            applets = self.currpanelsubject_settings.get_strv("applets")
            if self.uuid in applets:
                self.currpanelsubject_settings.connect(
                    "changed", self.check_ifonpanel
                )
        return False

    def check_ifonpanel(self, *args):
        applets = self.currpanelsubject_settings.get_strv("applets")
        self.countdown_onpanel = self.uuid in applets

    def get_currclaimedspace(self, *args):
        self.claimed_panelspace = self.space_settings.get_int("spacersize")
        if self.spacerpos == "top":
            self.containergrid.set_column_spacing(self.claimed_panelspace)
        else:
            self.containergrid.set_row_spacing(self.claimed_panelspace)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return CountDownSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True

    def do_panel_size_changed(self, panelsize, icsize, small_icsize):
        diff = icsize - small_icsize
        # 22 is the used icon size, 3 is an apparent constant
        self.panelspacing = round(
            ((panelsize - icsize) / 2) + ((icsize - 22) / 2) - 3
        )
        self.edit_grid()

    def do_panel_position_changed(self, panelposition):
        if panelposition != Budgie.PanelPosition.NONE:
            panelpositions = [
                Budgie.PanelPosition.RIGHT,
                Budgie.PanelPosition.LEFT,
                Budgie.PanelPosition.BOTTOM,
                Budgie.PanelPosition.TOP,
            ]
            angles = [270, 90, 0, 0]
            self.position_index = panelpositions.index(panelposition)
            if self.position_index > 1:
                self.labelpos = [2, 1, 1, 1]
                self.spacerpos = "top"
                self.timer.set_angle(0)
            else:
                self.labelpos = [1, 2, 1, 1]
                self.spacerpos = "left"
            self.timer.set_angle(angles[self.position_index])
            self.edit_grid()

    def edit_grid(self):
        for img in self.grid_helpers:
            self.containergrid.remove(img)
        spacerpath = "/usr/share/pixmaps/cr_panelspacer.png"
        self.grid_helpers = []
        spacerimg = Gtk.Image.new_from_file(spacerpath)
        built_in_spacer1 = Gtk.Image.new_from_file(spacerpath)
        built_in_spacer2 = Gtk.Image.new_from_file(spacerpath)
        for img in [
            spacerimg, built_in_spacer1, built_in_spacer2,
        ]:
            self.grid_helpers.append(img)
        if self.spacerpos == "left":
            self.containergrid.set_margin_left(3)
            self.containergrid.attach(spacerimg, 0, 1, 1, 1)
            self.containergrid.attach(built_in_spacer1, 1, 0, 1, 1)
            self.containergrid.attach(built_in_spacer2, 1, 2, 1, 1)
            self.containergrid.set_column_spacing(self.panelspacing)
            self.containergrid.set_row_spacing(self.claimed_panelspace)
        else:
            self.containergrid.set_margin_top(3)
            self.containergrid.attach(spacerimg, 1, 0, 1, 1)
            self.containergrid.attach(built_in_spacer1, 0, 1, 1, 1)
            self.containergrid.attach(built_in_spacer2, 2, 1, 1, 1)
            self.containergrid.set_row_spacing(self.panelspacing)
            self.containergrid.set_column_spacing(self.claimed_panelspace)
        # helper left or right, set size, add to self.grid_helpers
        self.panelgrid.remove(self.timer)
        if self.countdown != 0:
            self.panelgrid.attach(
                self.timer,
                self.labelpos[0],
                self.labelpos[1],
                self.labelpos[2],
                self.labelpos[3],
            )
        self.panelgrid.show_all()
        self.containergrid.show_all()

    def get_setting(self, checkbox, readval=False):
        file = os.path.join(settingspath, checkbox[1])
        exists = os.path.exists(file)
        if exists:
            val = open(file).read().strip()
            return exists, val
        else:
            return exists

    def manage_checkbox(self, button):
        newset = button.get_active()
        subject = [r for r in self.settingsdata if r[0] == button][0]
        runvar = subject[2]
        # set the runvar
        self.runvars[runvar] = newset
        # manage file
        file = os.path.join(settingspath, subject[1])
        if runvar in ["ringbell", "flashicon", "showwindow", "keeprun"]:
            if newset is False:
                open(file, "wt").write("")
            else:
                try:
                    os.remove(file)
                except FileNotFoundError:
                    pass
        elif runvar == "runcmd":
            self.command_entry.set_sensitive(newset)
            if newset is False:
                newfile = [False, self.command_entry.get_text()]
                open(file, "wt").write(str(newfile))

    def disable_suspend(self, filename, subkey):
        currval = powersettings.get_string(subkey)
        stored = os.path.join(settingspath, filename)
        open(stored, "wt").write(currval)
        powersettings.set_string(subkey, "nothing")

    def restore_prevset(self, filename, subkey):
        stored = os.path.join(settingspath, filename)
        try:
            prevset = open(stored).read().strip()
        except FileNotFoundError:
            print("failing to reset")
            pass
        else:
            powersettings.set_string(subkey, prevset)

    def two_dg(self, n):
        n = str(n)
        return "0" + n if len(n) == 1 else n

    def calc_timedisplay(self, seconds):
        hrs = self.two_dg(int(seconds / 3600))
        mins = self.two_dg(int((seconds % 3600) / 60))
        secs = self.two_dg(seconds % 60)
        return ":".join([hrs, mins, secs])

    def set_label(self, newlabel):
        GObject.idle_add(
            self.timer.set_text, newlabel,
            priority=GObject.PRIORITY_DEFAULT
        )

    def set_newicon(self, newicon):
        GObject.idle_add(
            self.seticon.set_from_pixbuf, newicon,
            priority=GObject.PRIORITY_DEFAULT
        )

    def lookup_stage(self, countdown):
        return [countdown == 0,
                0 < countdown <= self.red_time,
                self.red_time < countdown <= self.yellow_time,
                self.yellow_time < countdown,
                ].index(True)

    def end_signal(self):
        flash = self.runvars["flashicon"]
        ringbell = self.runvars["ringbell"]
        t = 0
        GObject.idle_add(
            self.applybutton.set_sensitive, False,
            priority=GObject.PRIORITY_DEFAULT
        )
        # set gsettings if self.runvars[keep_run]
        if self.runvars["keeprun"]:
            for s in subs:
                self.restore_prevset(s[0], s[1])
        if not self.cancel:
            if self.runvars["showwindow"]:
                subprocess.Popen([
                    "/usr/bin/zenity", "--info", '--title=CountDown message',
                    "--text=Count Down has ended"
                ])
            wait = any([flash, ringbell])
            bellcmd = [
                "/usr/bin/ogg123", "-q",
                "/usr/share/sounds/freedesktop/stereo/complete.oga"
            ]
            if self.runvars["runcmd"]:
                cmd = self.command_entry.get_text()
                self.run_command(cmd)

            while t < 6:
                if t % 2 == 0:
                    if ringbell:
                        subprocess.Popen(bellcmd)
                    if flash:
                        self.set_newicon(self.iconset[0])
                else:
                    if flash:
                        self.set_newicon(self.iconset[1])
                if wait:
                    time.sleep(0.7)
                t = t + 1
        for item in [
            self.set_state, self.applybutton.set_sensitive
        ]:
            GObject.idle_add(
                item, True,
                priority=GObject.PRIORITY_DEFAULT,
            )
        self.cancel = False

    def run_command(self, command):
        try:
            subprocess.Popen(["/bin/bash", "-c", command])
        except subprocess.CalledProcessError:
            pass

    def get_settime(self):
        return int(sum([
            self.hoursbutton.get_value() * 3600,
            self.minsbutton.get_value() * 60,
            self.secsbutton.get_value(),
        ]))

    def set_state(self, state, *args):
        for widget in [
            self.hoursbutton, self.minsbutton, self.secsbutton,
            self.sleep, self.nf_bell, self.runcomm,
            self.nf_icon, self.nf_message, self.command_entry,
            self.hrs_label, self.secs_label, self.mins_label,
        ]:
            widget.set_sensitive(state)
        if state is True:
            self.context_start.remove_class(Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION)
            self.context_start.add_class(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
            self.applybutton.set_label("Run")
            GObject.idle_add(
                self.panelgrid.remove, self.timer,
                priority=GObject.PRIORITY_DEFAULT,
            )
            GObject.idle_add(
                self.panelgrid.set_row_spacing, 10,
                priority=GObject.PRIORITY_DEFAULT,
            )
        else:
            self.applybutton.set_label("Stop")
            self.context_start.remove_class(Gtk.STYLE_CLASS_SUGGESTED_ACTION)
            self.context_start.add_class(Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION)
            GObject.idle_add(
                self.panelgrid.attach, self.timer,
                self.labelpos[0], self.labelpos[1],
                self.labelpos[2], self.labelpos[3],
                priority=GObject.PRIORITY_DEFAULT,
            )
            GObject.idle_add(
                self.panelgrid.set_row_spacing, 6,
                priority=GObject.PRIORITY_DEFAULT,
            )
        active = self.runcomm.get_active()
        active = False if not all([active, state]) else True
        GObject.idle_add(
            self.command_entry.set_sensitive, active,
            priority=GObject.PRIORITY_DEFAULT,
        )

    def handle_apply(self, button):
        set_t = self.get_settime()
        self.show_seconds = True
        if self.countdown != 0:
            # cancelling countdown
            self.cancel = True
            self.countdown, self.span = 0, 0
            timelabel = self.calc_timedisplay(self.countdown)
            self.set_label(timelabel)
        elif set_t != 0:
            # starting countdown, if set time > 0
            self.set_state(False)
            self.countdown = set_t + 1
            self.init_t = time.time()
            # time shift to the real time
            self.diff = 0
            self.span = self.countdown
            self.currstate1 = None
            # set gsettings if self.runvars[keep_run]
            if self.runvars["keeprun"]:
                for s in subs:
                    self.disable_suspend(s[0], s[1])

    def update_command(self, entry, *args):
        file = os.path.join(settingspath, "runcommand")
        b_state = self.runcomm.get_active()
        cmd = self.command_entry.get_text()
        newcmd = [b_state, cmd]
        open(file, "wt").write(str(newcmd))

    def run_countdown(self):
        cycle = 9
        self.currstate1 = None
        self.diff = 0
        while self.countdown_onpanel:
            time.sleep(1 - self.diff)
            # see where we are in the time stages
            self.currstate2 = self.lookup_stage(self.countdown)
            if self.currstate2 != self.currstate1:
                if self.currstate2 == 0:
                    self.end_signal()
                self.set_newicon(self.iconset[self.currstate2 - 1])
            if self.countdown != 0:
                self.countdown = self.countdown - 1
                timelabel = self.calc_timedisplay(self.countdown)
                if timelabel is not None:
                    self.set_label(timelabel)
                cycle = cycle + 1
                if cycle == 10:
                    # once per 10 seconds, synchronize with real clock
                    cycle = 0
                    realchange = time.time() - self.init_t
                    theo_change = self.span - self.countdown
                    self.diff = realchange - theo_change
                    self.diff = self.diff if self.diff <= 1 else 1
                else:
                    self.diff = 0
            self.currstate1 = self.currstate2

    def on_press(self, box, arg):
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import datetime
import errno
import os


class Log():

    def __init__(self, appConfigFolderName):

        self.TAG = "FileLog"
        self.logErrorOn = True
        self.logDebugOn = True
        self.logInfoOn = True

        self.ERROR_TAG = "Log e"
        self.DEBUG_TAG = "Log d"
        self.INFO_TAG = "Log i"

        dirPath = os.path.expanduser("~") + "/.cache/" + appConfigFolderName
        self.makeDirIfNotExist(dirPath)
        self.filePath = dirPath + "/log"

    # gets datatime
    def getDateTime(self):
        return datetime.datetime.now()

    # makes dir
    def makeDirIfNotExist(self, path):
        if (path != ""):
            try:
                os.makedirs(path)
            except OSError as e:
                if e.errno != errno.EEXIST:
                    raise

    ########################
    # Error log
    ########################
    def e(self, TAG, msg, e):
        if (self.logErrorOn):
            eArgsText = ""
            for arg in e.args:
                eArgsText = eArgsText + "%s " % arg
            logText = "\n[%s %s in %s] :\n%s\n%s\n%s\n" % \
                      (self.getDateTime(), self.ERROR_TAG, TAG, str(msg),
                       type(e).__name__, eArgsText)
            with open(self.filePath, "a+") as file:
                file.write(logText)

    ########################
    # Debug log
    ########################
    def d(self, TAG, msg):
        if (self.logDebugOn):
            logText = "\n[%s %s in %s] :\n%s\n" % \
                      (self.getDateTime(), self.DEBUG_TAG, TAG, str(msg))
            with open(self.filePath, "a+") as file:
                file.write(logText)

    ########################
    # İnfo log
    ########################
    def i(self, TAG, msg):
        if (self.logInfoOn):
            logText = "\n[%s %s in %s] :\n%s\n" % \
                      (self.getDateTime(), self.INFO_TAG, TAG, str(msg))
            with open(self.filePath, "a+") as file:
                file.write(logText)



#!/usr/bin/env python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import gi.repository

gi.require_version('Gtk', '3.0')
gi.require_version('GMenu', '3.0')
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie
from gi.repository import GMenu
from gi.repository import Gtk
from gi.repository import Gdk
from AppButton import AppButton
from PanelButton import PanelButton
from ArrowButton import ArrowButton
from EditButton import EditButton
from MenuButton import MenuButton
from DirectionalButton import DirectionalButton
from App import App
from SortHelper import SortHelper
from FilterHelper import FilterHelper
from LocaleHelper import LocaleHelper
from Log import Log
from Error import Error
from SelectButton import SelectButton


class AppLauncherApplet(Budgie.Applet):
    # Budgie.Applet is in fact a Gtk.Bin

    def __init__(self, uuid):
        self.TAG = "budgie-app-launcher.AppLauncher"
        self.APPINDICATOR_ID = "io_serdarsen_github_budgie_app_launcher"
        self.APPS_ID = "gnome-applications.menu"
        self.log = Log("budgie-app-launcher")
        self.sortHelper = SortHelper()
        self.filterHelper = FilterHelper()
        self.localHelper = LocaleHelper()
        self.iconSize = 24
        self.showOnPanel = 0
        self.tree = None  # GMenu.Tree
        self.appButtonsContainer = None  # Gtk.ListBox
        self.contentScroll = None  # Gtk.ScrolledWindow
        self.allAppsContentScroll = None
        self.currentMenuButton = None
        self.menuButtons = []
        self.allApps = []
        self.filteredActiveApps = []
        self.activeApps = []
        self.inactiveApps = []
        self.appLimitOnPanel = 10
        self.manager = None
        self.popover = None
        self.popoverHeight = 0
        self.popoverWidth = 300
        self.popoverHeight = 510
        Budgie.Applet.__init__(self)
        self.iconSize = self.localHelper.retriveIconSize()
        self.showOnPanel = self.localHelper.retriveShowOnPanel()
        self.buildIndicator()
        self.buildPopover()
        self.buildStack()
        self.loadAllApps()
        self.loadAppButtons()
        self.loadPanelButtons()
        self.vertical = False

    def do_panel_position_changed(self, position):
        # wait for signal, change orientation if it occurs
        check_or = any([
            position == Budgie.PanelPosition(pos) for pos in [8, 16]
        ])
        self.vertical = True if check_or else False
        self.reload_elements()

    def reload_elements(self):
        # on orientation change of the panel, set applet accordingly
        self.panelButtonsContainer.destroy()
        self.panelButtonsContainer = Gtk.VBox() if self.vertical \
            else Gtk.HBox()
        self.indicatorBox.add(self.panelButtonsContainer)
        self.update()

    ####################################
    # build START
    ####################################

    def buildIndicator(self):
        self.indicatorBox = Gtk.EventBox()
        self.panelButtonsContainer = Gtk.VBox()
        self.indicatorBox.add(self.panelButtonsContainer)
        self.add(self.indicatorBox)

    def buildPopover(self):
        self.popover = Budgie.Popover.new(self.indicatorBox)
        self.popover.set_default_size(self.popoverWidth, self.popoverHeight)
        self.popover.get_child().show_all()
        self.show_all()

    def buildStack(self):
        self.stack = Gtk.Stack()
        self.stack.set_homogeneous(False)
        self.stack.set_transition_type(
            Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
        self.popover.add(self.stack)
        self.buildStackPage1()
        self.buildStackPage2()

    def buildStackPage1(self):
        # page 1
        page1 = Gtk.Box(Gtk.Orientation.VERTICAL, 0)
        page1.border_width = 0
        self.stack.add_named(page1, "page1")
        page1.get_style_context().add_class("budgie-menu")
        # page 1 content
        page1InnerBox = Gtk.VBox()
        page1.pack_start(page1InnerBox, True, True, 0)
        titleBox = Gtk.HBox()
        page1InnerBox.pack_start(titleBox, False, False, 0)
        self.setMargins(titleBox, 3, 3, 3, 0)

        self.searchEntry = Gtk.SearchEntry()
        titleBox.pack_start(self.searchEntry, True, True, 0)
        self.searchEntry.connect("search-changed", self.searchEntryOnChange)
        self.searchEntry.connect("activate", self.searchEntryOnActivate)
        editButton = EditButton("Edit")
        editButton.connect("clicked", self.editButtonOnClick)
        titleBox.pack_end(editButton, False, False, 0)
        self.contentScroll = Gtk.ScrolledWindow(None, None)
        page1InnerBox.pack_start(self.contentScroll, True, True, 0)
        self.contentScroll.set_overlay_scrolling(True)
        self.contentScroll.set_policy(Gtk.PolicyType.NEVER,
                                      Gtk.PolicyType.AUTOMATIC)
        self.appButtonsContainer = Gtk.ListBox()
        self.contentScroll.add(self.appButtonsContainer)
        # self.content.row_activated.connect(on_row_activate)
        self.appButtonsContainer.set_selection_mode(Gtk.SelectionMode.NONE)
        # placeholder in case of no results
        placeholder = Gtk.Label("App Launcher")
        placeholder.use_markup = True
        placeholder.get_style_context().add_class("dim-label")
        placeholder.show()
        placeholder.margin = 6
        self.appButtonsContainer.valign = Gtk.Align.START
        self.appButtonsContainer.set_placeholder(placeholder)

    def buildStackPage2(self):
        # page 2
        page2 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        page2.border_width = 0
        self.setMargins(page2, 10, 3, 3, 3)
        self.stack.add_named(page2, "page2")

        # page 2 toolbar
        page2Toolbar = Gtk.HBox()
        page2.pack_start(page2Toolbar, False, False, 0)
        # page 2 toolbar content
        # cancel
        backButton = DirectionalButton("Back", Gtk.PositionType.LEFT)
        page2Toolbar.pack_start(backButton, True, True, 0)
        backButton.connect("clicked", self.backButtonOnClick)
        # up
        self.upButton = ArrowButton(Gtk.PositionType.TOP)
        page2Toolbar.pack_start(self.upButton, True, True, 0)
        self.upButton.connect("clicked", self.upButtonOnClick)
        self.upButton.set_sensitive(False)
        # down
        self.downButton = ArrowButton(Gtk.PositionType.BOTTOM)
        page2Toolbar.pack_start(self.downButton, True, True, 0)
        self.setMargins(page2Toolbar, 0, 3, 0, 0)
        self.downButton.connect("clicked", self.downButtonOnClick)
        self.downButton.set_sensitive(False)
        # done
        self.doneButton = Gtk.Button("Apply")
        self.doneButton.get_style_context().add_class("flat")
        page2Toolbar.pack_end(self.doneButton, True, True, 0)
        self.doneButton.connect("clicked", self.doneButtonOnClick)

        # page 2 toolbar 2
        page2Toolbar2 = Gtk.HBox()
        page2.pack_start(page2Toolbar2, False, False, 0)
        self.deselectButton = SelectButton(False)
        page2Toolbar2.pack_start(self.deselectButton, False, False, 0)
        self.deselectButton.addOnClickMethod(self.deselectButtonOnClick)
        self.selectButton = SelectButton(True)
        page2Toolbar2.pack_end(self.selectButton, False, False, 0)
        self.selectButton.addOnClickMethod(self.selectButtonOnClick)
        self.setMargins(page2Toolbar2, 0, 3, 0, 0)

        # page 2 allAppsContainer
        self.allAppsContainer = Gtk.VBox()
        page2.pack_start(self.allAppsContainer, True, True, 0)
        self.allAppsContentScroll = Gtk.ScrolledWindow(None, None)
        self.allAppsContainer.pack_start(self.allAppsContentScroll, True, True,
                                         0)
        self.allAppsContentScroll.set_overlay_scrolling(True)
        self.allAppsContentScroll.set_policy(Gtk.PolicyType.NEVER,
                                             Gtk.PolicyType.AUTOMATIC)
        self.menuButtonsContainer = Gtk.VBox()
        self.setMargins(self.menuButtonsContainer, 0, 0, 0, 7)
        self.allAppsContentScroll.add(self.menuButtonsContainer)
        # page 2 bottom bar
        page2BottomBar = Gtk.VBox()
        page2.pack_start(page2BottomBar, False, False, 0)
        self.setMargins(page2BottomBar, 10, 3, 3, 3)
        # page2BottomBar show panel Container
        page2BottomBarShowPanelContainer = Gtk.HBox()
        page2BottomBar.pack_start(page2BottomBarShowPanelContainer, False,
                                  False, 0)
        showOnPanelLabel = Gtk.Label("Show on panel", xalign=0)
        page2BottomBarShowPanelContainer.pack_start(showOnPanelLabel, True,
                                                    True, 0)
        showOnPanelLabel.get_style_context().add_class("dim-label")
        self.showOnPanelSpinButton = Gtk.SpinButton()
        self.showOnPanelSpinButton.set_adjustment(
            Gtk.Adjustment(0, 0, 900, 1, 10, 0))
        self.showOnPanelSpinButton.set_value(self.showOnPanel)
        page2BottomBarShowPanelContainer.pack_start(self.showOnPanelSpinButton,
                                                    False, False, 0)
        func = self.showOnPanelSpinButtonOnValueChange
        self.showOnPanelSpinButton.connect("value-changed",
                                           func)
        # page2BottomBar icon size Container
        page2BottomBarIconSizeContainer = Gtk.HBox()
        page2BottomBar.pack_start(page2BottomBarIconSizeContainer, False,
                                  False,
                                  0)
        iconSizeLabel = Gtk.Label("Icon size", xalign=0)
        page2BottomBarIconSizeContainer.pack_start(iconSizeLabel, True, True,
                                                   0)
        iconSizeLabel.get_style_context().add_class("dim-label")
        iconSizeSpinButton = Gtk.SpinButton()
        iconSizeSpinButton.set_adjustment(Gtk.Adjustment(0, 16, 512, 1, 10, 0))
        iconSizeSpinButton.set_value(self.iconSize)
        page2BottomBarIconSizeContainer.pack_start(
            iconSizeSpinButton, False, False, 0,
        )
        iconSizeSpinButton.connect(
            "value-changed", self.iconSizeSpinButtonOnValueChange,
        )

    def openStackPage1(self):
        self.stack.set_visible_child_name("page1")

    def openStackPage2(self):
        self.stack.set_visible_child_name("page2")

    def indicatorBoxOnPress(self, box, e):
        self.openStackPage1()
        if e.button != 1:
            return Gdk.EVENT_PROPAGATE
        if self.popover.get_visible():
            self.popover.hide()
        else:
            self.update()
            self.manager.show_popover(self.indicatorBox)
        return Gdk.EVENT_STOP

    def editButtonOnClick(self, editButton):
        self.currentMenuButton = None
        self.updateSensitiveUpDownButtons()
        self.doneButton.set_sensitive(False)
        self.update()
        self.openStackPage2()

    def upButtonOnClick(self, button):
        if self.currentMenuButton in self.menuButtons:
            newIndex = self.menuButtons.index(self.currentMenuButton) - 1

            if newIndex >= 0:
                self.menuButtonsContainer.reorder_child(self.currentMenuButton,
                                                        newIndex)
                self.menuButtons.remove(self.currentMenuButton)
                self.menuButtons.insert(newIndex, self.currentMenuButton)
                self.activeApps.remove(self.currentMenuButton.getApp())
                self.activeApps.insert(newIndex,
                                       self.currentMenuButton.getApp())
                self.allApps.remove(self.currentMenuButton.getApp())
                self.allApps.insert(newIndex, self.currentMenuButton.getApp())
                self.updateAllAppsIndexes()
            self.updateSensitiveUpDownButtons()

    def downButtonOnClick(self, button):
        if self.currentMenuButton in self.menuButtons:
            newIndex = self.menuButtons.index(self.currentMenuButton) + 1
            self.menuButtonsContainer.reorder_child(self.currentMenuButton,
                                                    newIndex)
            self.menuButtons.remove(self.currentMenuButton)
            self.menuButtons.insert(newIndex, self.currentMenuButton)
            self.activeApps.remove(self.currentMenuButton.getApp())
            self.activeApps.insert(newIndex, self.currentMenuButton.getApp())
            self.allApps.remove(self.currentMenuButton.getApp())
            self.allApps.insert(newIndex, self.currentMenuButton.getApp())
            self.updateAllAppsIndexes()
            self.updateSensitiveUpDownButtons()

    def backButtonOnClick(self, button):
        self.update()
        self.openStackPage1()

    def doneButtonOnClick(self, button):
        self.localHelper.saveApps(self.activeApps)
        self.loadAppButtons()
        self.loadPanelButtons()
        self.loadMenuButtons()
        self.allAppsContentScroll.get_vadjustment().set_value(0)
        self.doneButton.set_sensitive(False)
        self.showOnPanelSpinButton.get_adjustment().set_upper(
            len(self.activeApps))
        self.showOnPanelSpinButton.get_adjustment().set_value(self.showOnPanel)

    def selectButtonOnClick(self, selectButton):
        self.selectAll()

    def deselectButtonOnClick(self, deselectButton):
        self.deselectAll()

    def menuButtonOnToggle(self, toggleButton, *data):
        self.doneButton.set_sensitive(True)
        menuButton = data[0]
        # if current menu button toggled again set current None
        if self.currentMenuButton is not None and \
                self.currentMenuButton is menuButton:
            self.currentMenuButton = None
            self.updateSensitiveUpDownButtons()
            return
        # toggle false  old current button
        if self.currentMenuButton is not None and \
                self.currentMenuButton is not menuButton:
            self.currentMenuButton.setToggled(False)
        # set new currentMenuButton
        self.currentMenuButton = menuButton
        self.updateSensitiveUpDownButtons()

    def menuButtonOnCheck(self, checkButton, *data):
        self.doneButton.set_sensitive(True)
        menuButton = data[0]
        app = menuButton.getApp()
        if not app.getActive():
            app.setActive(True)
            if app in self.inactiveApps:
                self.inactiveApps.remove(app)
                self.activeApps.append(app)
        else:
            app.setActive(False)
            if app in self.activeApps:
                self.activeApps.remove(app)
                self.inactiveApps.insert(0, app)
        self.inactiveApps = self.sortHelper.sortedAppsByName(self.inactiveApps)
        self.allApps = self.activeApps + self.inactiveApps
        self.updateAllAppsIndexes()
        self.updateSensitiveSelectButtons()

    def showOnPanelSpinButtonOnValueChange(self, spinButton):
        # print("value changed : %s" % int(spinButton.get_value()))
        self.showOnPanel = int(spinButton.get_value())
        self.localHelper.saveShowOnPanel(self.showOnPanel)
        self.update()
        self.panelButtonsContainer.set_size_request(0, 0)
        spinButton.get_adjustment().set_upper(len(self.activeApps))

    def iconSizeSpinButtonOnValueChange(self, spinButton):
        # print("value changed : %s" % int(spinButton.get_value()))
        self.iconSize = int(spinButton.get_value())
        self.localHelper.saveIconSize(self.iconSize)
        self.update()

    def searchEntryOnChange(self, searchEntry):

        text = searchEntry.get_text().strip()

        self.filteredActiveApps = self.filterHelper.filteredAppsByName(
            self.activeApps, self.showOnPanel, text)

        if text == "":
            self.loadAppButtons()
        else:
            self.loadFilteredActiveAppButtons()

    def treeOnChange(self, tree):
        # self.log.d(self.TAG, "treeOnChange")
        self.tree = None
        self.update()
        self.localHelper.saveApps(self.activeApps)

    def searchEntryOnActivate(self, searchEntry):
        if len(self.filteredActiveApps) != 0:
            info = self.filteredActiveApps[0].getInfo()
            if info is not None:
                self.hidePopover()
                info.launch(None, None)

    def update(self):
        # self.popover.resize(self.popoverWidth, self.popoverHeight)
        self.searchEntry.set_text("")
        self.loadAllApps()
        self.loadAppButtons()
        self.loadMenuButtons()
        self.loadPanelButtons()
        self.updateSensitiveSelectButtons()
        self.popover.set_size_request(self.popoverWidth, self.popoverHeight)
        self.popover.get_child().show_all()
        self.show_all()

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.indicatorBox, self.popover)

    def updateSensitiveUpDownButtons(self):
        if self.currentMenuButton is not None:
            currentMenuButtonIndex = self.menuButtons.index(
                self.currentMenuButton)
            activeAppsLenght = len(self.activeApps)
            if currentMenuButtonIndex == 0:
                self.upButton.set_sensitive(False)
                self.downButton.set_sensitive(True)
            elif currentMenuButtonIndex is activeAppsLenght - 1:
                self.upButton.set_sensitive(True)
                self.downButton.set_sensitive(False)
            else:
                self.upButton.set_sensitive(True)
                self.downButton.set_sensitive(True)
        else:
            self.upButton.set_sensitive(False)
            self.downButton.set_sensitive(False)

    def updateSensitiveSelectButtons(self):
        if len(self.activeApps) == 0:
            self.selectButton.set_sensitive(True)
            self.deselectButton.set_sensitive(False)
        elif len(self.activeApps) > 0:
            self.selectButton.set_sensitive(True)
            self.deselectButton.set_sensitive(True)
        else:
            self.selectButton.set_sensitive(False)
            self.deselectButton.set_sensitive(True)

    def updateAllAppsIndexes(self):
        tempApps = []
        for app in self.allApps:
            app.setIndex(self.allApps.index(app))
            tempApps.append(app)
        self.allApps = tempApps

    def hidePopover(self):
        if (self.popover is not None):
            if self.popover.get_visible():
                self.popover.hide()

    def loadAllApps(self):
        # self.log.d(self.TAG, "loadAllApps")
        self.iconSize = self.localHelper.retriveIconSize()
        self.showOnPanel = self.localHelper.retriveShowOnPanel()
        # reset all apps lists
        self.allApps = []
        self.activeApps = []
        self.inactiveApps = []
        # load our active apps data from local
        self.activeAppsDict = self.localHelper.retriveAppsDict()
        # load all apps from system all applications
        appsDict = {}
        self.loadAndExtractApps(appsDict)
        # sort and combine active and inactiveApps to build allApps
        self.activeApps = self.sortHelper.sortedAppsByIndex(self.activeApps)
        self.inactiveApps = self.sortHelper.sortedAppsByName(self.inactiveApps)
        self.allApps = self.activeApps + self.inactiveApps

    def loadAndExtractApps(self, appsDict, treeRoot=None):
        # self.log.d(self.TAG, "loadAndExtractApps")
        if self.tree is None:
            # self.log.d(self.TAG, "loadAndExtractApps self.tree is None")
            self.tree = GMenu.Tree.new(self.APPS_ID,
                                       GMenu.TreeFlags.SORT_DISPLAY_NAME)
            self.tree.connect("changed", self.treeOnChange)
            try:
                self.tree.load_sync()
            except Exception as e:
                self.log.e(self.TAG, Error.ERROR_8011, e)
        if treeRoot is None:
            root = self.tree.get_root_directory()
        else:
            root = treeRoot
        it = None
        if root is not None:
            it = root.iter()
        if it is not None:
            while True:
                treeItemType = it.next()
                if treeItemType is GMenu.TreeItemType.INVALID:
                    break
                if treeItemType is GMenu.TreeItemType.DIRECTORY:
                    dir = it.get_directory()
                    # self.log.d(self.TAG, "loadAppList dir %s : " % dir)
                    self.loadAndExtractApps(appsDict, dir)
                elif treeItemType is GMenu.TreeItemType.ENTRY:
                    info = it.get_entry().get_app_info()
                    # self.log.d(self.TAG, info.get_display_name())
                    id = info.get_id()
                    if id not in appsDict:
                        appsDict[id] = info
                        if id in self.activeAppsDict:
                            app = self.activeAppsDict[id]
                            app.setId(id)
                            app.setInfo(info)
                            self.activeApps.insert(app.getIndex(), app)
                        else:
                            app = App(id, info.get_display_name(), False)
                            app.setInfo(info)
                            self.inactiveApps.append(app)

    def loadFilteredActiveAppButtons(self):
        if self.appButtonsContainer is not None:  # empty allAppsContent
            for appButton in self.appButtonsContainer.get_children():
                appButton.destroy()
        for app in self.filteredActiveApps:
            row = Gtk.HBox()
            self.appButtonsContainer.add(row)
            appButton = AppButton(app, 24, self.popover)
            row.pack_start(appButton, True, True, 0)
            appButton.show_all()
            row.show_all()
        self.appButtonsContainer.show_all()

    def loadAppButtons(self):
        if self.appButtonsContainer is not None:  # empty allAppsContent
            for appButton in self.appButtonsContainer.get_children():
                appButton.destroy()
        counter = 0
        for app in self.allApps:
            if counter >= self.showOnPanel and app.getActive():
                row = Gtk.HBox()
                self.appButtonsContainer.add(row)
                appButton = AppButton(app, 24, self.popover)
                row.pack_start(appButton, True, True, 0)
                appButton.show_all()
                row.show_all()
            counter += 1
        self.appButtonsContainer.show_all()

    def loadPanelButtons(self):
        if self.panelButtonsContainer is not None:  # empty allAppsContent
            for panelButton in self.panelButtonsContainer.get_children():
                panelButton.destroy()
        # add the applet's button as first item
        appletbutton = Gtk.Button()
        applet_icon = Gtk.Image.new_from_icon_name(
            "budgie-app-launcher-applet-symbolic", Gtk.IconSize.MENU
        )
        appletbutton.set_image(applet_icon)
        appletbutton.set_relief(Gtk.ReliefStyle.NONE)
        self.panelButtonsContainer.add(appletbutton)
        appletbutton.connect(
            "button-press-event", self.indicatorBoxOnPress
        )
        counter = 0
        for app in self.allApps:
            if counter < self.showOnPanel and app.getActive():
                panelButton = PanelButton(app, self.iconSize, self.popover)
                self.panelButtonsContainer.add(panelButton)
            counter += 1
        self.panelButtonsContainer.show_all()

    def loadMenuButtons(self):
        self.menuButtons = []
        if self.menuButtonsContainer is not None:
            for menuButton in self.menuButtonsContainer.get_children():
                menuButton.destroy()
        for app in self.allApps:
            menuButton = MenuButton(app, 24)
            menuButton.setChecked(app.getActive())
            menuButton.setToggButtonSensitive(app.getActive())
            menuButton.addOnToggleMethod(self.menuButtonOnToggle)
            menuButton.addOnCheckMethod(self.menuButtonOnCheck)
            menuButton.show_all()
            self.menuButtons.append(menuButton)
            self.menuButtonsContainer.add(menuButton)
        self.menuButtonsContainer.show_all()

    def setMargins(self, widget, top, bottom, left, right):
        widget.set_margin_top(top)
        widget.set_margin_bottom(bottom)
        widget.set_margin_left(left)
        widget.set_margin_right(right)

    def selectAll(self):
        self.doneButton.set_sensitive(True)
        for menuButton in self.menuButtons:
            if not menuButton.getChecked():
                menuButton.setChecked(True)

    def deselectAll(self):
        self.doneButton.set_sensitive(True)
        for menuButton in self.menuButtons:
            if menuButton.getChecked():
                menuButton.setChecked(False)



#!/usr/bin/env python3

# standard includes
import sys

# dbus includes
import dbus

# project includes

# well-known name for our program
ECHO_BUS_NAME = 'org.UbuntuBudgie.ExtrasDaemon'

# interfaces implemented by some objects in our program
ECHO_INTERFACE = 'org.UbuntuBudgie.ExtrasDaemon'

# paths to some objects in our program
ECHO_OBJECT_PATH = '/org/ubuntubudgie/extrasdaemon'


def client(mes):
    bus = dbus.SessionBus()

    try:
        proxy = bus.get_object(ECHO_BUS_NAME, ECHO_OBJECT_PATH)
    except dbus.DBusException as e:
        # There are actually two exceptions thrown:
        # 1: org.freedesktop.DBus.Error.NameHasNoOwner
        #   (when the name is not registered by any running process)
        # 2: org.freedesktop.DBus.Error.ServiceUnknown
        #   (during auto-activation since there is no .service file)
        # TODO figure out how to suppress the activation attempt
        # also, there *has* to be a better way of managing exceptions
        if e._dbus_error_name != \
                'org.freedesktop.DBus.Error.ServiceUnknown':
            raise
        if e.__context__._dbus_error_name != \
                'org.freedesktop.DBus.Error.NameHasNoOwner':
            raise
        print('client: No one can hear me!!')
    else:
        iface = dbus.Interface(proxy, ECHO_INTERFACE)

        iface.ResetLayout(mes)


def main(exe, args):
    if args:
        client(' '.join(args))
    else:
        sys.exit('Usage: %s message...' % exe)


if __name__ == '__main__':
    main(sys.argv[0], sys.argv[1:])



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


class Error():
    ERROR_1010 = "Error:1010"
    ERROR_1011 = "Error:1011"

    ERROR_3010 = "Error:3010"

    ERROR_8010 = "Error:8010"
    ERROR_8011 = "Error:8011"



import gi.repository
import time
import subprocess
import os
gi.require_version('Budgie', '1.0')
from gi.repository import Budgie, GObject, Gtk, Gio, GLib
import os


"""
Budgie TakeaBreak
Author: Jacob Vlijm
Copyright © 2017 Ubuntu Budgie Developers
Website=https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""


tmp = os.getenv("XDG_RUNTIME_DIR") \
    if "XDG_RUNTIME_DIR" in os.environ else os.getenv("HOME")
nextbreakfile = os.path.join(tmp, ".nextbreak")
tab_settings = Gio.Settings.new("org.ubuntubudgie.plugins.takeabreak")


class BudgieTakeaBreak(GObject.GObject, Budgie.Plugin):
    """ This is simply an entry point into your Budgie Applet implementation.
        Note you must always override Object, and implement Plugin.
    """

    # Good manners, make sure we have unique name in GObject type system
    __gtype_name__ = "BudgieTakeaBreak"

    def __init__(self):
        """ Initialisation is important.
        """
        GObject.Object.__init__(self)

    def do_get_panel_widget(self, uuid):
        """ This is where the real fun happens. Return a new Budgie.Applet
            instance with the given UUID. The UUID is determined by the
            BudgiePanelManager, and is used for lifetime tracking.
        """
        return BudgieTakeaBreakApplet(uuid)


class BudgieTakeaBreakSettings(Gtk.Grid):
    def __init__(self, setting):

        super().__init__()
        self.setting = setting
        # maingrid
        maingrid = Gtk.Grid()
        maingrid.set_column_homogeneous(False)
        maingrid.set_border_width(10)
        self.add(maingrid)
        # uptime section
        uptime_label = Gtk.Label("Work time (min):\t", xalign=0)
        maingrid.attach(uptime_label, 0, 0, 1, 1)
        uptime_set = tab_settings.get_int("awaketime")
        uptime = Gtk.SpinButton.new_with_range(1, 90, 1)
        uptime.set_value(uptime_set)
        uptime.connect("value-changed", self.update_setting, "awaketime")
        maingrid.attach(uptime, 1, 0, 1, 1)
        # breaktime section
        breaktime_label = Gtk.Label("Break time (min):\t", xalign=0)
        maingrid.attach(breaktime_label, 0, 1, 1, 1)
        breaktime_set = tab_settings.get_int("sleeptime")
        breaktime = Gtk.SpinButton.new_with_range(1, 90, 1)
        breaktime.set_value(breaktime_set)
        breaktime.connect("value-changed", self.update_setting, "sleeptime")
        maingrid.attach(breaktime, 1, 1, 1, 1)
        # sep below time section
        maingrid.attach(Gtk.Label("\n"), 0, 5, 1, 1)
        # show notifications checkbox
        shownotify = Gtk.CheckButton("Show notifications")
        notify_set = tab_settings.get_boolean("showmessage")
        shownotify.set_active(notify_set)
        shownotify.connect("toggled", self.update_setting, "showmessage")
        maingrid.attach(shownotify, 0, 6, 4, 1)
        # smart resume checkbox
        smartres = Gtk.CheckButton("Smart resume")
        smartres_set = tab_settings.get_boolean("smartresume")
        smartres.set_active(smartres_set)
        smartres.connect("toggled", self.update_setting, "smartresume")
        smartres.set_tooltip_text(
            "After a break, start count down when user is active"
        )
        maingrid.attach(smartres, 0, 7, 4, 1)
        # automatically unlock checkbox
        autounlock = Gtk.CheckButton("Automatically unlock after break")
        autounlock_set = tab_settings.get_boolean("unlockafterbreak")
        autounlock.set_active(autounlock_set)
        autounlock.connect("toggled", self.update_setting, "unlockafterbreak")
        autounlock.set_tooltip_text(
            "After a break, automatically unlock screen"
        )
        maingrid.attach(autounlock, 0, 8, 4, 1)
        # sep below  section
        maingrid.attach(Gtk.Label("\n"), 0, 9, 1, 1)
        # option label
        breaktime_label = Gtk.Label("Effect:\n", xalign=0)
        maingrid.attach(breaktime_label, 0, 10, 1, 1)
        # dropdown
        self.effect_options = [
            ["rotate", "Screen upside down"],
            ["dim", "Dim screen"],
            ["message", "Countdown message"],
            ["lock", "Lock screen"],
        ]
        effect_set = tab_settings.get_string("mode")
        effect_index = [s[0] for s in self.effect_options].index(effect_set)
        effect_box = Gtk.Box()
        command_combo = Gtk.ComboBoxText()
        command_combo.set_entry_text_column(0)
        for cmd in self.effect_options:
            command_combo.append_text(cmd[1])
        command_combo.set_active(effect_index)
        command_combo.connect("changed", self.update_setting, "mode")
        maingrid.attach(effect_box, 0, 11, 2, 1)
        effect_box.pack_start(command_combo, False, True, 0)
        self.show_all()

    def update_setting(self, widget, setting):
        if setting in ["awaketime", "sleeptime"]:
            newval = int(widget.get_value())
            tab_settings.set_int(setting, newval)
        elif setting in ["showmessage", "smartresume", "unlockafterbreak"]:
            newval = widget.get_active()
            tab_settings.set_boolean(setting, newval)
        elif setting == "mode":
            newval = self.effect_options[widget.get_active()][0]
            tab_settings.set_string(setting, newval)


class BudgieTakeaBreakApplet(Budgie.Applet):
    """ Budgie.Applet is in fact a Gtk.Bin """

    def __init__(self, uuid):

        self.tab_message = ""
        Budgie.Applet.__init__(self)
        self.uuid = uuid
        appletpath = os.path.dirname(os.path.abspath(__file__))
        self.runnerpath = os.path.join(appletpath, "takeabreak_run")
        # setup exit-from-panel triggers
        self.currpanelsubject_settings = None
        GLib.timeout_add_seconds(1, self.watchout)
        # applet appearance
        self.icon = Gtk.Image()
        self.img_normal = "takeabreak-symbolic"
        self.img_normal = "takeabreakpaused-symbolic"
        self.icon.set_from_icon_name(
            "takeabreak-symbolic", Gtk.IconSize.MENU
        )
        self.box = Gtk.EventBox()
        self.box.add(self.icon)
        self.add(self.box)
        self.popover = Budgie.Popover.new(self.box)
        self.maingrid = Gtk.Grid()
        self.popover.add(self.maingrid)
        self.maingrid.attach(Gtk.Label("\t"), 0, 0, 1, 1)
        self.maingrid.attach(Gtk.Label("\t"), 100, 100, 1, 1)
        self.next_label = Gtk.Label("Next break")
        self.time_label = Gtk.Label("")
        self.next_label.set_xalign(0.5)
        self.time_label.set_xalign(0.5)
        self.maingrid.attach(self.next_label, 1, 1, 1, 1)
        self.maingrid.attach(self.time_label, 1, 2, 1, 1)
        tab_settings.connect("changed", self.test)
        self.on_offbutton = Gtk.Switch()
        self.onoff_set = tab_settings.get_boolean("paused")
        # run runner if False
        self.switched_on = self.onoff_set is False
        if self.switched_on:
            self.on_offbutton.set_active(True)
        else:
            self.on_offbutton.set_active(False)
            self.icon.set_from_icon_name(
                "takeabreakpaused-symbolic", Gtk.IconSize.MENU
            )
        self.maingrid.attach(self.on_offbutton, 1, 3, 1, 1)
        self.on_offbutton.connect("state-set", self.act_on_switch)
        self.maingrid.show_all()
        self.box.show_all()
        self.show_all()
        self.box.connect("button-press-event", self.on_press)
        GLib.timeout_add_seconds(2, self.reset_app, self.onoff_set is False)

    def watchout(self):
        path = "com.solus-project.budgie-panel"
        panelpath_prestring = "/com/solus-project/budgie-panel/panels/"
        panel_settings = Gio.Settings.new(path)
        allpanels_list = panel_settings.get_strv("panels")
        for p in allpanels_list:
            panelpath = panelpath_prestring + "{" + p + "}/"
            self.currpanelsubject_settings = Gio.Settings.new_with_path(
                path + ".panel", panelpath
            )
            applets = self.currpanelsubject_settings.get_strv("applets")
            if self.uuid in applets:
                self.currpanelsubject_settings.connect(
                    "changed", self.check_ifonpanel
                )
        return False

    def check_ifonpanel(self, *args):
        applets = self.currpanelsubject_settings.get_strv("applets")
        if self.uuid not in applets:
            self.kill_runner()

    def test(self, *args):
        self.reset_app(tab_settings.get_boolean("paused") is False)

    def act_on_switch(self, button, state):
        tab_settings.set_boolean("paused", state is False)
        self.reset_app(state)
        # if switched on/of from the popup, calculate the time here
        # instead of reading it from the file (else timing issue on startup)
        if state:
            self.set_time_fromapplet()
            self.icon.set_from_icon_name(
                "takeabreak-symbolic", Gtk.IconSize.MENU
            )
        else:
            self.icon.set_from_icon_name(
                "takeabreakpaused-symbolic", Gtk.IconSize.MENU
            )

    def set_time_fromapplet(self):
        wait = tab_settings.get_int("awaketime") * 60
        currt = time.time()
        newtime = time.strftime("%H:%M", time.localtime(wait + currt))
        self.time_label.set_text(newtime)

    def set_labels(self, state):
        if state:
            try:
                popuptime = open(nextbreakfile).read().split(".")[0]
                ltime = time.localtime(int(popuptime))
                newtime = time.strftime(
                    "%H:%M", time.localtime(int(popuptime))
                )
                self.time_label.set_text(newtime)
            except (FileNotFoundError, UnboundLocalError):
                self.set_time_fromapplet()
        else:
            self.time_label.set_text(".  .  .")

    def reset_app(self, state):
        self.set_labels(state)
        self.kill_runner()
        if state:
            subprocess.Popen(self.runnerpath)
        return False

    def kill_runner(self):
        user = os.environ["USER"]
        try:
            # I know, old school, but it works well
            pid = subprocess.check_output([
                "/usr/bin/pgrep", "-f", "-u", user, self.runnerpath,
            ]).decode("utf-8").strip()
            subprocess.Popen(["/usr/bin/kill", pid])
        except subprocess.CalledProcessError:
            pass

    def on_press(self, box, arg):
        curr_paused = tab_settings.get_boolean("paused")
        self.set_labels(curr_paused is False)
        self.manager.show_popover(self.box)

    def do_update_popovers(self, manager):
        self.manager = manager
        self.manager.register_popover(self.box, self.popover)

    def do_get_settings_ui(self):
        """Return the applet settings with given uuid"""
        return BudgieTakeaBreakSettings(self.get_applet_settings(self.uuid))

    def do_supports_settings(self):
        """Return True if support setting through Budgie Setting,
        False otherwise.
        """
        return True



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import Gdk
from Log import Log
from Error import Error


class PanelButton(Gtk.Button):

    def __init__(self, app, iconSize, popover):
        Gtk.Button.__init__(self)
        self.popover = popover
        self.TAG = "budgie-app-launcher.PanelButton"
        self.log = Log("budgie-app-launcher")
        self.app = app
        self.info = self.app.getInfo()  # DesktopAppInfo
        self.get_style_context().add_class("flat")
        self.isActionsAvailable = False
        self.menu = None
        img = Gtk.Image.new_from_gicon(self.info.get_icon(),
                                       Gtk.IconSize.INVALID)
        img.set_pixel_size(iconSize)
        self.add(img)
        # self.set_tooltip_text(parent.get_description())
        self.set_tooltip_text(self.info.get_display_name())
        self.connect("button-press-event", self.onPress)
        self.buildMenu()

    def showMenu(self, button, event):
        if self.isActionsAvailable:
            self.menu.popup_at_pointer(event)

    def buildMenu(self):
        self.menu = Gtk.Menu()
        actions = self.info.list_actions()

        if len(actions) != 0:
            self.isActionsAvailable = True
            for action in actions:
                displayName = self.info.get_action_name(action)
                item = Gtk.MenuItem.new_with_label(displayName)
                item.connect("activate", self.itemActivate, action)
                item.show_all()
                self.menu.append(item)

    def itemActivate(self, item, *data):
        action = data[0]
        if action is None:
            return
        self.hidePopover()
        self.info.launch_action(action, None)

    def getApp(self):
        return self.app

    def getInfo(self):
        return self.info

    def hidePopover(self):
        if (self.popover is not None):
            if self.popover.get_visible():
                self.popover.hide()

    def onPress(self, button, event):
        if event.button != 1:
            self.showMenu(button, event)
            return Gdk.EVENT_PROPAGATE
        else:
            self.hidePopover()
            self.launchApp()
        return Gdk.EVENT_STOP

    def launchApp(self):
        try:
            self.info.launch(None, None)
        except Exception as e:
            self.log.e(self.TAG, Error.ERROR_8010, e)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class EditButton(Gtk.Button):

    def __init__(self, toolTipText):
        Gtk.Button.__init__(self)
        self.border_width = 0
        self.set_can_focus(False)
        # self.set_tooltip_text(toolTipText)
        self.get_style_context().add_class("flat")
        self.image = Gtk.Image.new_from_icon_name(
            "budgie-app-launcher-edit-symbolic", Gtk.IconSize.MENU)
        self.add(self.image)



#!/usr/bin/python3

# This file is part of App Launcher

# Copyright © 2018 Ubuntu Budgie Developers

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.


import gi.repository

gi.require_version('Gtk', '3.0')
from gi.repository import Gtk


class DirectionalButton(Gtk.Button):

    def __init__(self, label_str, arrow_direction):

        Gtk.Button.__init__(self)

        box = Gtk.Box(Gtk.Orientation.HORIZONTAL, 0)
        box.halign = Gtk.Align.FILL
        label = Gtk.Label(label_str)
        image = Gtk.Image()

        if arrow_direction is Gtk.PositionType.RIGHT:
            image.set_from_icon_name("go-next-symbolic", Gtk.IconSize.MENU)
            box.pack_start(label, True, True, 0)
            box.pack_end(image, False, False, 1)
            image.margin_start = 6
            label.margin_start = 6
        else:
            image.set_from_icon_name("go-previous-symbolic",
                                     Gtk.IconSize.MENU)
            box.pack_start(image, False, False, 0)
            box.pack_start(label, True, True, 0)
            image.margin_end = 6

        label.halign = Gtk.Align.START
        label.margin = 0
        box.margin = 0
        box.border_width = 0
        self.get_style_context().add_class(Gtk.STYLE_CLASS_FLAT)
        self.add(box)



import os
from Xlib.display import Display
from Xlib import X
from Xlib.ext.xtest import fake_input
import Xlib.XK


"""
Budgie QuickChar
Author: Jacob Vlijm
Copyright © 2017-2019 Ubuntu Budgie Developers
Website: https://ubuntubudgie.org
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or any later version. This
program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
A PARTICULAR PURPOSE. See the GNU General Public License for more details. You
should have received a copy of the GNU General Public License along with this
program.  If not, see <http://www.gnu.org/licenses/>.
"""


def paste_char(*args, **kwargs):
    for c in args:
        fake_input(_display, X.KeyPress, _to_keysim(c))
        _display.sync()
    for c in reversed(args):
        fake_input(_display, X.KeyRelease, _to_keysim(c))
        _display.sync()


def _to_keysim(gotkey):
    return _display.keysym_to_keycode(Xlib.XK.string_to_keysym(gotkey))


_display = Display(os.environ['DISPLAY'])
